from functools import update_wrapper

from django.conf.urls import url
from django.contrib import admin

from davinci.caldav.forms import CalDAVServerForm, CalDAVCalendarWizard, SelectCalDAVServerForm, \
    SelectCalDAVCalendarForm
from davinci.caldav.models import CalDAVServer, CalDAVCalendar


class CalDAVServerAdmin(admin.ModelAdmin):
    form = CalDAVServerForm
    list_display = ['name', 'url', 'username', 'ssl_verify']


class CalDAVCalendarAdmin(admin.ModelAdmin):
    list_display = ['name', 'server', 'calendar_url']

    def get_urls(self):
        urls = super(CalDAVCalendarAdmin, self).get_urls()
        opts = self.model._meta
        context = {
            'title': f'Add {opts.verbose_name}',
            'current_app': self.admin_site.name,
            'add': True,
            'opts': opts,
            'app_label': opts.app_label,
        }
        view = CalDAVCalendarWizard.as_view(
            extra_context=context,
            form_list=[SelectCalDAVServerForm, SelectCalDAVCalendarForm]
        )

        def wrap(view):
            def wrapper(*args, **kwargs):
                kwargs['admin'] = self
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        add_url = url(r"add/$", wrap(view), name="formwizard_add")
        change_url = url(r"^(.+)/change/$", wrap(view), name="formwizard_change")
        return [add_url, change_url] + urls

admin.site.register(CalDAVServer, CalDAVServerAdmin)
admin.site.register(CalDAVCalendar, CalDAVCalendarAdmin)
