from typing import List

import caldav
from django import forms
from formtools.wizard.views import SessionWizardView

from davinci.caldav.models import CalDAVServer, CalDAVCalendar


class CalDAVServerForm(forms.ModelForm):
    class Meta:
        model = CalDAVServer
        widgets = {
            'password': forms.PasswordInput
        }
        fields = "__all__"


class SelectCalDAVServerForm(forms.ModelForm):
    class Meta:
        model = CalDAVCalendar
        fields = ['server']


class SelectCalDAVCalendarForm(forms.ModelForm):
    class Meta:
        model = CalDAVCalendar
        fields = ['calendar_url']


class CalDAVCalendarWizard(SessionWizardView):
    template_name = "forms/calendar_wizard.html"

    def dispatch(self, request, *args, admin=None, **kwargs):
        self._model_admin = admin
        return super(CalDAVCalendarWizard, self).dispatch(request, *args, **kwargs)

    def get_form(self, step=None, data=None, files=None):
        form = super(CalDAVCalendarWizard, self).get_form(step, data, files)
        if step is None:
            step = self.steps.current

        # Add choices data to calendar ID field on step 1
        if step == '1':
            d = self.get_cleaned_data_for_step('0')
            cals: List[caldav.Calendar] = d['server'].get_calendars()
            existing_cals = [x.calendar_url for x in CalDAVCalendar.objects.filter(server=d['server'])]
            cals = [x for x in cals if x.url not in existing_cals]
            form.fields['calendar_url'] = forms.ChoiceField(choices=[
                (x.url, x.name) for x in cals
            ])

        return form

    def done(self, form_list, form_dict, **kwargs):
        server = form_dict['0'].cleaned_data['server']
        url = form_dict['1'].cleaned_data['calendar_url']
        calendar = CalDAVCalendar.objects.create(server=server, calendar_url=url,
                                                 name=server.get_calendar_name(url))
        return self._model_admin.response_add(self.request, calendar)
