import logging

from django.core.management.base import BaseCommand
from django.utils import timezone

from davinci.icalendar.decorators import handle_lock
from davinci.icalendar.models import ICalSync


class Command(BaseCommand):
    help = 'Sychronize iCal syncs that need to be synchronized.'

    @handle_lock
    def handle(self, *args, **options):
        logger = logging.getLogger("davinci.icalendar.ical_sync")
        logger.debug(f"Command `ical_sync` invoked")

        for sync in ICalSync.objects.filter(active=True):
            sync: ICalSync
            if sync.last_sync is None or timezone.now() > (sync.last_sync + sync.sync_interval):
                logger.debug(f"Synchronizing {sync.name}...")

                events = sync.get_events()
                added, existing, deleted = sync.target.upload_events(events, purge=sync.purge)

                sync.last_sync = timezone.now()
                sync.save()
                logger.debug(f"Sync for {sync.name} done! {added} events added, {existing} updated, and {deleted} purged.")

        self.stdout.write("iCal sync complete!")
